# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Jun  6 2014)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc
import wxx
import wx.richtext

###########################################################################
## Class GitView
###########################################################################

class GitView ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 451,470 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer2 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer2.AddGrowableCol( 0 )
		fgSizer2.AddGrowableRow( 0 )
		fgSizer2.SetFlexibleDirection( wx.BOTH )
		fgSizer2.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_tree = wxx.TreeCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TR_HAS_BUTTONS|wx.TR_HIDE_ROOT|wx.TR_SINGLE )
		fgSizer2.Add( self.m_tree, 0, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer2 )
		self.Layout()
	
	def __del__( self ):
		pass
	

###########################################################################
## Class NewGitRepo
###########################################################################

class NewGitRepo ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Create new Git repository", pos = wx.DefaultPosition, size = wx.Size( 415,286 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer229 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer229.AddGrowableCol( 0 )
		fgSizer229.AddGrowableRow( 0 )
		fgSizer229.SetFlexibleDirection( wx.BOTH )
		fgSizer229.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_choicebook7 = wx.Choicebook( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.CHB_DEFAULT )
		self.m_panel30 = wx.Panel( self.m_choicebook7, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		fgSizer233 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer233.AddGrowableCol( 0 )
		fgSizer233.AddGrowableRow( 1 )
		fgSizer233.SetFlexibleDirection( wx.BOTH )
		fgSizer233.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_dirPicker2 = wx.DirPickerCtrl( self.m_panel30, wx.ID_ANY, wx.EmptyString, u"Select a folder", wx.DefaultPosition, wx.DefaultSize, wx.DIRP_DIR_MUST_EXIST|wx.DIRP_USE_TEXTCTRL )
		fgSizer233.Add( self.m_dirPicker2, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_textCtrl77 = wx.TextCtrl( self.m_panel30, wx.ID_ANY, u"Select the directory in what you want to create a new local git repository. It may be a directory already containing files. This case, you must add these files to the git control after the creation.", wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE|wx.TE_READONLY|wx.NO_BORDER )
		self.m_textCtrl77.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOWFRAME ) )
		
		fgSizer233.Add( self.m_textCtrl77, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		self.m_panel30.SetSizer( fgSizer233 )
		self.m_panel30.Layout()
		fgSizer233.Fit( self.m_panel30 )
		self.m_choicebook7.AddPage( self.m_panel30, u"local repository", False )
		fgSizer229.Add( self.m_choicebook7, 1, wx.EXPAND |wx.ALL, 5 )
		
		fgSizer159 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer159.AddGrowableCol( 1 )
		fgSizer159.SetFlexibleDirection( wx.BOTH )
		fgSizer159.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer159.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer12 = wx.StdDialogButtonSizer()
		self.m_sdbSizer12OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer12.AddButton( self.m_sdbSizer12OK )
		self.m_sdbSizer12Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer12.AddButton( self.m_sdbSizer12Cancel )
		m_sdbSizer12.Realize();
		
		fgSizer159.Add( m_sdbSizer12, 1, wx.EXPAND, 5 )
		
		
		fgSizer229.Add( fgSizer159, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer229 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer12Cancel.Bind( wx.EVT_BUTTON, self.OnCancel )
		self.m_sdbSizer12OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnCancel( self, event ):
		event.Skip()
	
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class OpenGitRepo
###########################################################################

class OpenGitRepo ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Open Git repository", pos = wx.DefaultPosition, size = wx.Size( 351,163 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer229 = wx.FlexGridSizer( 3, 1, 0, 0 )
		fgSizer229.AddGrowableCol( 0 )
		fgSizer229.AddGrowableRow( 1 )
		fgSizer229.SetFlexibleDirection( wx.BOTH )
		fgSizer229.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_dirPicker2 = wx.DirPickerCtrl( self, wx.ID_ANY, wx.EmptyString, u"Select a folder", wx.DefaultPosition, wx.DefaultSize, wx.DIRP_DIR_MUST_EXIST|wx.DIRP_USE_TEXTCTRL )
		fgSizer229.Add( self.m_dirPicker2, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_textCtrl77 = wx.TextCtrl( self, wx.ID_ANY, u"Select the directory that holds a git repository", wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE|wx.TE_READONLY|wx.NO_BORDER )
		self.m_textCtrl77.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_WINDOWFRAME ) )
		
		fgSizer229.Add( self.m_textCtrl77, 0, wx.ALL|wx.ALIGN_CENTER_HORIZONTAL|wx.EXPAND, 5 )
		
		fgSizer159 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer159.AddGrowableCol( 1 )
		fgSizer159.SetFlexibleDirection( wx.BOTH )
		fgSizer159.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer159.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer12 = wx.StdDialogButtonSizer()
		self.m_sdbSizer12OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer12.AddButton( self.m_sdbSizer12OK )
		self.m_sdbSizer12Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer12.AddButton( self.m_sdbSizer12Cancel )
		m_sdbSizer12.Realize();
		
		fgSizer159.Add( m_sdbSizer12, 1, wx.EXPAND, 5 )
		
		
		fgSizer229.Add( fgSizer159, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer229 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer12Cancel.Bind( wx.EVT_BUTTON, self.OnCancel )
		self.m_sdbSizer12OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnCancel( self, event ):
		event.Skip()
	
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class CommitGit
###########################################################################

class CommitGit ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Commit to Git", pos = wx.DefaultPosition, size = wx.Size( 350,247 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer229 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer229.AddGrowableCol( 0 )
		fgSizer229.AddGrowableRow( 0 )
		fgSizer229.SetFlexibleDirection( wx.BOTH )
		fgSizer229.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		sbSizer53 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"commit message" ), wx.VERTICAL )
		
		self.m_richText39 = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0|wx.VSCROLL|wx.HSCROLL|wx.NO_BORDER|wx.WANTS_CHARS )
		sbSizer53.Add( self.m_richText39, 1, wx.EXPAND |wx.ALL, 5 )
		
		
		fgSizer229.Add( sbSizer53, 1, wx.EXPAND|wx.ALL, 5 )
		
		fgSizer159 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer159.AddGrowableCol( 1 )
		fgSizer159.SetFlexibleDirection( wx.BOTH )
		fgSizer159.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer159.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer12 = wx.StdDialogButtonSizer()
		self.m_sdbSizer12OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer12.AddButton( self.m_sdbSizer12OK )
		self.m_sdbSizer12Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer12.AddButton( self.m_sdbSizer12Cancel )
		m_sdbSizer12.Realize();
		
		fgSizer159.Add( m_sdbSizer12, 1, wx.EXPAND, 5 )
		
		
		fgSizer229.Add( fgSizer159, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer229 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer12Cancel.Bind( wx.EVT_BUTTON, self.OnCancel )
		self.m_sdbSizer12OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnCancel( self, event ):
		event.Skip()
	
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class NewGitRemote
###########################################################################

class NewGitRemote ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"New remote ...", pos = wx.DefaultPosition, size = wx.Size( 475,193 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer229 = wx.FlexGridSizer( 3, 1, 0, 0 )
		fgSizer229.AddGrowableCol( 0 )
		fgSizer229.AddGrowableRow( 0 )
		fgSizer229.SetFlexibleDirection( wx.BOTH )
		fgSizer229.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		fgSizer65 = wx.FlexGridSizer( 3, 2, 0, 0 )
		fgSizer65.AddGrowableCol( 1 )
		fgSizer65.SetFlexibleDirection( wx.BOTH )
		fgSizer65.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText26 = wx.StaticText( self, wx.ID_ANY, u"name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText26.Wrap( -1 )
		fgSizer65.Add( self.m_staticText26, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		self.m_name = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer65.Add( self.m_name, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticText27 = wx.StaticText( self, wx.ID_ANY, u"url", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText27.Wrap( -1 )
		fgSizer65.Add( self.m_staticText27, 0, wx.ALL|wx.ALIGN_RIGHT, 5 )
		
		self.m_url = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer65.Add( self.m_url, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticText28 = wx.StaticText( self, wx.ID_ANY, u"password", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText28.Wrap( -1 )
		fgSizer65.Add( self.m_staticText28, 0, wx.ALL, 5 )
		
		self.m_password = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_PASSWORD )
		fgSizer65.Add( self.m_password, 0, wx.ALL|wx.EXPAND, 5 )
		
		
		fgSizer229.Add( fgSizer65, 1, wx.EXPAND|wx.TOP|wx.RIGHT|wx.LEFT, 5 )
		
		fgSizer159 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer159.AddGrowableCol( 1 )
		fgSizer159.SetFlexibleDirection( wx.BOTH )
		fgSizer159.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer159.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer12 = wx.StdDialogButtonSizer()
		self.m_sdbSizer12OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer12.AddButton( self.m_sdbSizer12OK )
		self.m_sdbSizer12Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer12.AddButton( self.m_sdbSizer12Cancel )
		m_sdbSizer12.Realize();
		
		fgSizer159.Add( m_sdbSizer12, 1, wx.EXPAND, 5 )
		
		
		fgSizer229.Add( fgSizer159, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer229 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer12Cancel.Bind( wx.EVT_BUTTON, self.OnCancel )
		self.m_sdbSizer12OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnCancel( self, event ):
		event.Skip()
	
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class Progress
###########################################################################

class Progress ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Git progress", pos = wx.DefaultPosition, size = wx.Size( 445,278 ), style = wx.CAPTION|wx.STAY_ON_TOP )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer12 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer12.AddGrowableCol( 0 )
		fgSizer12.AddGrowableRow( 0 )
		fgSizer12.SetFlexibleDirection( wx.BOTH )
		fgSizer12.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_text = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.TE_MULTILINE|wx.TE_READONLY )
		fgSizer12.Add( self.m_text, 0, wx.ALL|wx.EXPAND, 5 )
		
		fgSizer159 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer159.AddGrowableCol( 1 )
		fgSizer159.SetFlexibleDirection( wx.BOTH )
		fgSizer159.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer159.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer12 = wx.StdDialogButtonSizer()
		self.m_sdbSizer12OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer12.AddButton( self.m_sdbSizer12OK )
		m_sdbSizer12.Realize();
		
		fgSizer159.Add( m_sdbSizer12, 1, wx.EXPAND, 5 )
		
		
		fgSizer12.Add( fgSizer159, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer12 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer12OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnOK( self, event ):
		event.Skip()
	




# -*- coding: utf-8 -*-
import wx
from deco import classproperty


class context(object):
    """context is a required class for dealing between datamodel
    and rendering"""
    _app = None
    _config = None

    def __init__(self):
        """init"""
        pass

    @classproperty
    def app(cls):
        """return the app"""
        return cls._app

    @app.setter
    def setapp(cls, app):
        """set the app"""
        cls._app = app
        cls.load()

    @classmethod
    def load(cls):
        """do first app initialization"""
        pass

    @classmethod
    def save(cls):
        """do app status save"""
        pass

    @classproperty
    def config(cls):
        """Get config"""
        return cls._config

    @config.setter
    def setconfig(cls, config):
        """Set config"""
        cls._config = config

    @classproperty
    def mru(cls):
        """Get the mru"""
        return cls._mru

    @classmethod
    def RenderUndoRedoAdd(cls, obj):
        """render undo/redo"""
        raise "not implemented"

    @classmethod
    def RenderLoadedAdd(cls, obj):
        """render loaded add"""
        raise "not implemented"

    @classmethod
    def RenderUndoRedoRemoving(cls, obj):
        """render undo/redo"""
        raise "not implemented"

    @classmethod
    def RenderUndoRedoChanged(cls, obj):
        """render undo/redo"""
        raise "not implemented"

    @classmethod
    def Destroy(cls):
        """Destroy context"""
        cls.save()
        cls.frame.Destroy()


class localcontext(context):
    """context for doing local works"""
    _frame = None

    @classproperty
    def frame(cls):
        """accessor for frame"""
        return cls._frame

    def __init__(self):
        """init"""
        super(localcontext, self).__init__()

    @classmethod
    def load(_):
        """Starts the config (may be started after set app)
           cctx : context class
           app  : application object"""
        config = wx.Config('beatle.local', vendorName='melvm',
            style=wx.CONFIG_USE_LOCAL_FILE)
        config.SetPath('~/.config')
        _.config = config
        #
        _._mru = wx.FileHistory()
        _._mru.Load(_.config)
        _.frame.LoadViews()
        _.app.LoadSession()
        _.frame.LoadMRU()

    @classmethod
    def save(_):
        """Save context"""
        mru = _.mru
        mru.Save(_.config)
        _.frame.SaveViews()
        _.app.SaveSession()
        _.config.Flush()

    @classmethod
    def SetFrame(cctx, frame):
        """Set frame window"""
        cctx._frame = frame

    @classmethod
    def SaveProject(cctx, proj):
        """Save all changes"""
        book = cctx.frame.docBook
        for i in range(0, book.GetPageCount()):
            pane = book.GetPage(i)
            if hasattr(pane, '_object'):
                if pane._object.project == proj:
                    pane.Commit()
        cctx.app.SaveProject(proj)

    @classmethod
    def RenderUndoRedoAdd(cctx, obj):
        """render undo/redo"""
        #update old models
        if not hasattr(obj, '_visibleInTree'):
            obj._visibleInTree = True
        #end old models
        if obj._visibleInTree:
            cctx.frame.DoRenderAddElement(obj)

    @classmethod
    def RenderLoadedAdd(cctx, obj):
        """render loaded add"""
        #update old models
        if not hasattr(obj, '_visibleInTree'):
            obj._visibleInTree = True
        #end old models
        if obj._visibleInTree:
            cctx.frame.DoRenderAddElement(obj)

    @classmethod
    def RenderUndoRedoRemoving(cctx, obj):
        """render undo/redo"""
        if obj._visibleInTree:
            cctx.frame.DoRenderRemoveElement(obj)

    @classmethod
    def RenderUndoRedoChanged(cctx, obj):
        """render undo/redo"""
        cctx.frame.UpdateElement(obj)




"""Subclass of NewVariable, which is generated by wxFormBuilder."""

import re
import copy
import wx
import wxx
from activity.models.ui import ui as ui
import model
import app.resources as rc


# Implementing NewVariable
class VariableDialog(ui.NewVariable):
    """
    This dialog allows to setup a non-member
    variable.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Dialog initialization"""
        super(VariableDialog, self).__init__(parent)
        self._container = container
        self._types = dict([(x._name, x) for x in container.types])
        # add types but not on-the-fly template type
        self.m_choice1.AppendItems([x for x in self._types.keys() if x != '@'])
        self.choiceStr = ""
        self.m_textCtrl6.SetFocus()
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("member"))
        self.SetIcon(icon)

    # Handlers for NewVariable events.
    def OnTypeChanged(self, event):
        """Handle type change event"""
        """This event happens when the return type is changed. The main goal
        of this callback is handling template types for argument specification"""
        iSel = self.m_choice1.GetCurrentSelection()
        _type = self._types.get(self.m_choice1.GetString(iSel), None)
        template_args = False
        if _type is not None:
            if _type._template is not None:
                template_args = True
        if template_args is True:
            self.m_staticText67.Enable(True)
            self.m_template_args.Enable(True)
            self.m_staticText68.Enable(True)
        else:
            self.m_staticText67.Enable(False)
            self.m_template_args.Enable(False)
            self.m_staticText68.Enable(False)
            self.m_template_args.SetValue('')
        event.Skip()

    def CopyAttributes(self, variable):
        """Get the atributes"""
        variable._name = self._name
        variable._typei = copy.copy(self._typei)
        variable._static = self._static
        variable._default = self._default
        variable._volatile = self._volatile
        variable.note = self._note

    def SetAttributes(self, member):
        """Set the attributes"""
        self.m_textCtrl6.SetValue(member._name)
        ti = member._typei
        iSel = self.m_choice1.FindString(ti.type_name)
        self.m_choice1.SetSelection(iSel)
        iSel = self.m_choice2.FindString(member._access)
        self.m_choice2.SetSelection(iSel)
        self.m_checkBox105.SetValue(member._static)
        self.m_textCtrl8.SetValue(member._default)
        self.m_checkBox49.SetValue(member._volatile)
        self.m_checkBox11.SetValue(ti._const)
        self.m_checkBox13.SetValue(ti._ptr)
        self.m_checkBox12.SetValue(ti._ref)
        self.m_checkBox15.SetValue(ti._ptr_to_ptr)
        self.m_checkBox14.SetValue(ti._const_ptr)
        self.m_checkBox17.SetValue(ti._array)
        if ti._array is True:
            self.m_textCtrl7.Show(True)
            self.m_textCtrl7.Enable(True)
            self.m_textCtrl7.SetValue(str(ti._array_size))
        else:
            self.m_textCtrl7.SetValue('0')
        if ti._type_args is not None:
            self.m_staticText67.Enable(True)
            self.m_template_args.Enable(True)
            self.m_staticText68.Enable(True)
            self.m_template_args.SetValue(ti._type_args)
        self.m_richText1.SetValue(member.note)
        self.SetTitle("Edit variable")

    def Validate(self):
        """Dialog validation"""
        self._name = self.m_textCtrl6.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Variable name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Variable name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        iSel = self.m_choice1.GetCurrentSelection()
        if iSel == wx.NOT_FOUND:
            wx.MessageBox("Invalid type", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        typename = self.m_choice1.GetString(iSel)
        self._static = self.m_checkBox105.IsChecked()
        self._default = self.m_textCtrl8.GetValue()
        self._volatile = self.m_checkBox49.GetValue()
        if self.m_checkBox17.IsChecked():
            try:
                asize = int(self.m_textCtrl7.GetValue())
            except:
                asize = ''
        else:
            asize = None
        _type = self._types[typename]
        if _type._template is not None:
            #we construct type instance with explicit arguments
            type_args = self.m_template_args.GetValue()
            self._typei = model.cc.typeinst(
                type=_type,
                type_args=type_args,
                const=self.m_checkBox11.IsChecked(),
                ptr=self.m_checkBox13.IsChecked(),
                ref=self.m_checkBox12.IsChecked(),
                ptrptr=self.m_checkBox15.IsChecked(),
                constptr=self.m_checkBox14.IsChecked(),
                array=self.m_checkBox17.IsChecked(),
                arraysize=asize
            )
        else:
            self._typei = model.cc.typeinst(
                type=self._types[typename],
                const=self.m_checkBox11.IsChecked(),
                ptr=self.m_checkBox13.IsChecked(),
                ref=self.m_checkBox12.IsChecked(),
                ptrptr=self.m_checkBox15.IsChecked(),
                constptr=self.m_checkBox14.IsChecked(),
                array=self.m_checkBox17.IsChecked(),
                arraysize=asize
        )
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """return arguments for object instance"""
        return  {'parent': self._container, 'name': self._name,
            'type': self._typei, 'static': self._static,
            'volatile': self._volatile, 'default': self._default,
            'note': self._note}

    def OnKeyDown(self, event):
        """Listbox selection"""
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_UP or keycode == wx.WXK_NUMPAD_UP:
            i = self.m_choice1.GetSelection()
            if i is not wx.NOT_FOUND and i > 0:
                self.m_choice1.SetSelection(i - 1)
        elif keycode == wx.WXK_DOWN or keycode == wx.WXK_NUMPAD_DOWN:
            i = self.m_choice1.GetSelection() + 1
            if i > wx.NOT_FOUND and i < len(self._types):
                self.m_choice1.SetSelection(i)
        elif keycode < 256:
            keychar = chr(keycode)
            if keychar.isalnum() or keycode is wx.WXK_SPACE:
                self.choiceStr += keychar.lower()
                for t in self._types:
                    tl = t.lower()
                    if tl.find(self.choiceStr) == 0:
                        sel = self.m_choice1.FindString(t)
                        if sel is not wx.NOT_FOUND:
                            self.m_choice1.SetSelection(sel)
                            if keycode is not wx.WXK_SPACE:
                                event.Skip()
                            return
            self.choiceStr = ""
        event.Skip()

    def OnToggleStatic(self, event):
        """toggle static event"""
        event.Skip()

    def OnPointerToggle(self, event):
        """ptr toggle gui"""
        if self.m_checkBox13.IsChecked():
            self.m_checkBox14.Enable(True)
            self.m_checkBox50.Enable(True)
            self.m_checkBox15.Enable(True)
        else:
            self.m_checkBox14.Enable(False)
            self.m_checkBox50.Enable(False)
            self.m_checkBox15.Enable(False)
            self.m_checkBox14.SetValue(False)
            self.m_checkBox50.SetValue(False)
            self.m_checkBox15.SetValue(False)
        event.Skip()

    def OnToggleArray(self, event):
        """Handle toggle array"""
        if  self.m_checkBox17.IsChecked():
            self.m_textCtrl7.Show(True)
            self.m_textCtrl7.Enable(True)
        else:
            self.m_textCtrl7.Show(False)
            self.m_textCtrl7.Enable(False)
        event.Skip()

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 23:24:30 2013

@author: mel
"""
import model
import app.resources as rc


def name(x):
    """comparison"""
    return x._name


class LibrariesFolder(model.Folder):
    """Implements a Folder representation"""
    class_container = False
    type_container = False
    function_container = False
    variable_container = False
    module_container = False
    namespace_container = False
    function_container = False
    member_container = False

    #visual methods
    def draggable(self):
        """returns info about if the object can be moved"""
        return False

    def __init__(self, **kwargs):
        """Initialization"""
        kwargs['name'] = 'Libraries'
        super(LibrariesFolder, self).__init__(**kwargs)
        self.update_container()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        return super(LibrariesFolder, self).get_kwargs()

    def update_container(self):
        """Update the container info"""
        self.context_container = False
        self.folder_container = False
        self.diagram_container = False
        self.namespace_container = False

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folderP")

    def ExistMemberDataNamed(self, name):
        """Check recursively about the existence of nested child member"""
        return False




# -*- coding: utf-8 -*-
from FindText import FindText
from Folder import FolderDialog
from FindInFiles import FindInFilesDialog
from ImportProject import ImportProjectDialog
from Wait import WaitDialog
from BuildTools import BuildTools
from CodeNavigator import CodeNavigator
from Workspace import WorkspaceDialog
from Preferences import PreferencesDialog
from Working import WorkingDialog




# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""
import git
import wx
import model
import os
import os.path
import copy
import app.resources as rc
import app.ui.dlg as dlg
from ctx import localcontext as context


class GitRepo(model.TComponent):
    """Implements git repo representation"""
    folder_container = True

    def __init__(self, **kwargs):
        """Initialization method"""
        self._lastHdrTime = None
        self._lastSrcTime = None
        self._local = kwargs.get('local', True)
        self._uri = kwargs.get('uri', None)
        if not kwargs.get('name', None):
            s = self._uri
            s, t = os.path.split(s)
            if len(t) == 0:
                s, t = os.path.split(s)
            kwargs['name'] = t
        super(GitRepo, self).__init__(**kwargs)
        # The first step is to iterate over the git repository
        # structure to fill it
        working = dlg.WorkingDialog(context.frame)
        working.Show(True)
        wx.YieldIfNeeded()
        self._repo = git.Repo(self._uri)
        model.git.GitRemotes(parent=self)
        self.load_dir()
        working.Close()

    @property
    def stage_size(self):
        """return the number of elements in stage"""
        return len(self._repo.index.entries)

    def commit(self, message):
        """do a commit"""
        self._repo.index.commit(message)
        self._repo.index.write()
        self.update_status()

    def add(self, element):
        """add element to repository"""
        self._repo.index.add([element.rpath])
        self._repo.index.write()
        self.update_status()

    @property
    def repo(self):
        """return the model"""
        return self

    def load_dir(self):
        """Load the head commit and add each entry to repository view"""
        hdir = os.path.realpath(self.path)
        if not os.access(hdir, os.R_OK):
            return False
        kwargs = {'parent': self}
        for elem in os.listdir(hdir):
            if elem[0] == '.':
                continue
            path = os.path.join(hdir, elem)
            if os.path.isdir(path):
                kwargs = {'name': elem, 'parent': self}
                model.git.GitDir(**kwargs)
            if os.path.isfile(path):
                kwargs = {'name': elem, 'parent': self}
                model.git.GitFile(**kwargs)
            else:
                continue

    def status(self, path):
        """return the status of the file"""
        status = 'file'
        #check first if the file is under control
        if path not in self._repo.untracked_files:
            status = 'git_file'
            modified = self._repo.index.diff(None)
            for f in modified:
                if path not in [f.a_path, f.b_path]:
                    continue
                #ok, the file is modified
                if not f.b_mode and not f.b_blob and not f.b_path:
                    return 'git_file_deleted'
                else:
                    status = 'git_file_modified'
                    break
            staged = self._repo.index.diff('HEAD')
            for f in staged:
                if path not in [f.a_path, f.b_path]:
                    continue
                #ok, the file is modified
                if not f.b_mode and not f.b_blob and not f.b_path:
                    return 'git_file_deleted'
                else:
                    status = 'git_file_staged'
                    break
        return status

    @property
    def path(self):
        """Access for this element"""
        return self._uri

    @property
    def rpath(self):
        """Access for this element"""
        return ''

    def __getstate__(self):
        """Set pickle context"""
        # The repo object has not pickle support ...
        state = copy.copy(self.__dict__)
        del state['_repo']
        return state

    def __setstate__(self, d):
        """Load pickle context"""
        # The repo object has not pickle support ...
        self.__dict__ = d
        self._repo = git.Repo(self._uri)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {
            'local': self._local,
            'uri': self._uri}
        kwargs.update(super(GitRepo, self).get_kwargs())
        return kwargs

    def update_status(self):
        """update the repository status"""
        wx.YieldIfNeeded()
        hdir = os.path.realpath(self.path)
        if not os.access(hdir, os.R_OK):
            return
        dirs = []
        files = []
        for elem in os.listdir(hdir):
            path = os.path.join(hdir, elem)
            if elem[0] == '.':
                continue
            if os.path.isdir(path):
                dirs.append(elem)
            if os.path.isfile(path):
                files.append(elem)
            else:
                continue
        #current
        curdirs = self[model.git.GitDir]
        curfiles = self[model.git.GitFile]
        #new
        newdirs = [x for x in dirs if x not in [y.name for y in curdirs]]
        newfiles = [x for x in files if x not in [y.name for y in curfiles]]
        #delete dirs and files
        deldir = [x for x in curdirs if x.name not in dirs]
        delfile = [x for x in curfiles if x.name not in files]
        for x in deldir:
            x.Delete()
        for x in delfile:
            x.Delete()
        #update existing
        for d in self[model.git.GitDir]:
            d.update_status()
        for f in self[model.git.GitFile]:
            f.update_status()
            f.OnUndoRedoChanged()
        #new files and dirs
        for elem in newdirs:
            model.git.GitDir(name=elem, parent=self)
        for elem in newfiles:
            model.git.GitFile(name=elem, parent=self)
        #update remote status
        for element in self[model.git.GitRemotes]:
            element.update_status()

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(GitRepo, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        super(GitRepo, self).Delete()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(GitRepo, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(GitRepo, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        super(GitRepo, self).SaveState()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        super(GitRepo, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        #when the class is updated, ctor's and dtors, must be updated
        super(GitRepo, self).OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(GitRepo, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('git_repo')

    @property
    def label(self):
        """Get tree label"""
        return 'repository {self._name}'.format(self=self)






# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""
import model
import app.resources as rc
from tran import TransactionStack


class Inheritance(model.TComponent):
    """Implements class representation"""
    def __init__(self, **kwargs):
        """ Initialice the inheritance. Required parameters:
                ancestor: the parent class
            parent: the child class
            Optional parameters are:
               name: the name of the inheritance. This parameter is
            really not expected, because the inheritance cannot be aliased."""
        assert 'ancestor' in kwargs
        assert 'parent' in kwargs
        self._ancestor = kwargs['ancestor']
        self._access = kwargs.get('access', "public")
        self._virtual = kwargs.get('virtual', False)
        if 'name' not in kwargs:
            scoped = lambda x: (hasattr(x, 'scoped') and x.scoped) or x.name
            kwargs['name'] = scoped(self._ancestor)
        super(Inheritance, self).__init__(**kwargs)
        self._ancestor._deriv.append(self.parent)
        self.parent.AutoInit()
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        dias = self.project(model.ClassDiagram)
        for dia in dias:
            elem = dia.FindElement(self)
            if elem is not None:
                dia.SaveState()
                dia.RemoveElement(elem)
            if hasattr(dia, '_pane') and dia._pane is not None:
                dia._pane.Refresh()
        k = self.outer_class or self.outer_module
        super(Inheritance, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['ancestor'] = self._ancestor
        kwargs['access'] = self._access
        kwargs['virtual'] = self._virtual
        kwargs.update(super(Inheritance, self).get_kwargs())
        return kwargs

    @property
    def virtual_bases(self):
        """Gets the list of virtual bases through inheritance"""
        r = self._ancestor.virtual_bases
        if self._virtual and self._ancestor not in r:
            r.append(self._ancestor)
        return r

    def SetAncestor(self, ancestor):
        """Update ancestor"""
        if self._ancestor != ancestor:
            if not self._ancestor is None:
                self._ancestor._deriv.remove(self.parent)
        self._ancestor = ancestor
        if not self._ancestor is None:
            self._ancestor._deriv.append(self.parent)
            self._name = self._ancestor._name

    def RemoveRelations(self):
        """Utility for undo/redo"""
        self._ancestor._deriv.remove(self.parent)
        super(Inheritance, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        self._ancestor._deriv.append(self.parent)
        super(Inheritance, self).RestoreRelations()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        if root and not TransactionStack.InUndoRedo():
            project = self.project
            if project is not None:
                # remove from diagrams
                dias = project(model.ClassDiagram)
                for dia in dias:
                    # Check if inherit is in
                    elem = dia.FindElement(self)
                    if elem is not None:
                        TransactionStack.DoSaveState(dia)
                        dia.RemoveElement(elem)
        super(Inheritance, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Reflect changes"""
        for x in self.parent._deriv:
            x.OnUndoRedoChanged()
        super(Inheritance, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        #self._ancestor._deriv.append(self.parent)
        if not TransactionStack.InUndoRedo():
            # Check class diagrams for update
            project = self.project
            if project is not None:
                dias = project(model.ClassDiagram)
                for dia in dias:
                    # If inheritance is already, skip
                    if dia.FindElement(self) is not None:
                        continue
                    # Check if both classes are represented in
                    parent = dia.FindElement(self._ancestor)
                    if parent is None:
                        continue
                    child = dia.FindElement(self.inner_class)
                    if child is None:
                        continue
                    # Inheritance must be added to transaction
                    # (even when hidden)
                    TransactionStack.DoSaveState(dia)
                    dia.AddInheritance(self, parent, child)
        super(Inheritance, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("inheritance", self._access)

    @property
    def tree_label(self):
        """Get tree label"""
        return "inherits " + self._name





"""Subclass of TextPane, which is generated by wxFormBuilder."""

import wx
import tran
from app.ui.pane import BasePane
from activity.files.ui import ui as ui


# Implementing TextPane
class TextPane(BasePane, ui.TextPane):
    """Implements text editor"""

    def __init__(self, parent, mainframe, fileobj):
        """Intialization of text editor"""
        self._parent = parent
        self._mainframe = mainframe
        self._object = fileobj
        super(TextPane, self).__init__(parent)
        self._defaulBkColor = self.GetBackgroundColour()
        self.m_editor.SetText(fileobj.GetText())
        self.m_editor.HandleBookmarks(fileobj.bookmarks)
        self.m_editor.EnableBookmarks(True)
        self.m_editor.Initialize()
        self.m_editor.SetReadOnly(fileobj._readOnly)
        self.Layout()

    def Refresh(self):
        """Update editor from external changes (like undo/redo)"""
        oldText = self.m_editor.GetText()
        newText = self._object.GetText()
        if oldText == newText:
            return
        line = self.m_editor.GetFirstVisibleLine()
        pos = self.m_editor.GetCurrentPos()
        self.m_editor.Freeze()
        self.m_editor.SetText(self._object.GetText())
        self.m_editor.SetModified(False)
        try:
            self.m_editor.SetFirstVisibleLine(line)
            self.m_editor.SetCurrentPos(pos)
        except:
            pass
        self.m_editor.Thaw()

    def OnEditorChange(self, event):
        """Called when editor status change"""
        if self.m_editor.GetModify():
            self.SetBackgroundColour(wx.LIGHT_GREY)
        else:
            self.SetBackgroundColour(self._defaulBkColor)

    @tran.TransactionalMethod('modify file {}')
    def Commit(self):
        """Do commit changes"""
        if self.m_editor.GetModify():
            self._object.SaveState()
            self._object.SetText(self.m_editor.GetText())
            self.m_editor.SetModified(False)
            self.SetBackgroundColour(self._defaulBkColor)
            return True
        else:
            return False

    @property
    def line(self):
        """return current line"""
        return self.m_editor.GetCurrentLine()

    def OnDelete(self, event):
        """Handle delete key"""
        (b, e) = self.m_editor.GetSelection()
        if b == e:
            self.m_editor.CharRight()
        self.m_editor.DeleteBack()

    def OnUpdateSave(self, event):
        """Handles update event"""
        event.Enable(self.m_editor.GetModify())

    def OnSave(self, event):
        """Handles save event"""
        self.Commit()





# -*- coding: utf-8 -*-

import app.ui.pane as pane


class WebBrowserPane(pane.NavigatorPane):
    """Implements web browser pane"""
    def __init__(self, parent, mainframe):
        """Intialization of method editor"""
        super(WebBrowserPane, self).__init__(parent, mainframe)
        self.m_url.Show(True)

    def OnEnterUrl(self, event):
        """Process enter text"""
        self._url = self.m_url.GetValue()
        self.m_page.LoadURL(self._url)





# -*- coding: utf-8 -*-

"""
Some utilities for c++ parsing
"""

import re


def clean_code(text):
    """This method removes comments and closed statements."""
    delimiters = [('/*', '*/'), ('{', '}')]
    for delimiter in delimiters:
        test = True
        while test:
            test = False
            right_pos = text.find(delimiter[1])
            if right_pos > 0:
                left_pos = text.rfind(delimiter[0], 0, right_pos)
                if left_pos >= 0:
                    text = text[:left_pos] + '0' + text[right_pos + 1:]
                    test = True
    #quit newlines and line comments
    return ''.join((x + ' ')[0:x.find('//')] for x in text.splitlines())

# some definitions
TYPE_KEYWORD = r"bool|char|char16_t|char32_t|double|float|int|long|short|signed|unsigned|void|wchar_t"
OTHER_KEYWORD = r"alignas|alignof|and|and_eq|asm|auto|"\
          r"bitand|bitor|break|case|catch|class|"\
          r"compl|const|constexpr|const_cast|continue|decltype|default|"\
          r"delete|do|dynamic_cast|else|enum|explicit|export|extern|"\
          r"false|for|final|friend|goto|if|inline|mutable|namespace|"\
          r"new|noexcept|not|not_eq|nullptr|operator|or|or_eq|private|"\
          r"protected|public|register|reinterpret_cast|return|signed|"\
          r"sizeof|static|static_assert|static_cast|struct|switch|template|"\
          r"this|thread_local|throw|true|try|typedef|typeid|typename|"\
          r"union|unsigned|using|virtual|volatile|while|xor|xor_eq"

KEYWORD = r'{TYPE_KEYWORD}|{OTHER_KEYWORD}'.format(
    TYPE_KEYWORD=TYPE_KEYWORD, OTHER_KEYWORD=OTHER_KEYWORD)
FIRST_ID_CHAR = r'[_A-Za-z]'
OTHER_ID_CHAR = r'[_A-Za-z0-9]'
IDENTIFIER = r'(?<!{OTHER_ID_CHAR}){FIRST_ID_CHAR}+{OTHER_ID_CHAR}*'.format(FIRST_ID_CHAR=FIRST_ID_CHAR, OTHER_ID_CHAR=OTHER_ID_CHAR)
UNSIGNED = r'[0-9]+'
INTEGER = r'[\+-]?{UNSIGNED}'.format(UNSIGNED=UNSIGNED)
POSITIVE = r'\+?{UNSIGNED}'.format(UNSIGNED=UNSIGNED)
FLOAT = r'{INTEGER}\.{UNSIGNED}'.format(INTEGER=INTEGER, UNSIGNED=UNSIGNED)
NUMBER = r'{INTEGER}|{FLOAT}'.format(INTEGER=INTEGER, FLOAT=FLOAT)
STRING = r'"[^"]*"'
CHAR = r'\'[^\']*\''
ARRAY = r'\{[^\}]*\}'
CONSTANT = r'{CHAR}|{STRING}|{NUMBER}|{ARRAY}'.format(CHAR=CHAR,
    STRING=STRING, NUMBER=NUMBER, ARRAY=ARRAY)
NAMESPACE = '(?!{KEYWORD}){IDENTIFIER}'.format(KEYWORD=KEYWORD, IDENTIFIER=IDENTIFIER)
TYPENAME = '(?!{OTHER_KEYWORD}){IDENTIFIER}'.format(OTHER_KEYWORD=OTHER_KEYWORD, IDENTIFIER=IDENTIFIER)
VARNAME = '(?!{KEYWORD}){IDENTIFIER}'.format(KEYWORD=KEYWORD, IDENTIFIER=IDENTIFIER)
MEMBERNAME = '(?!{KEYWORD}){IDENTIFIER}'.format(KEYWORD=KEYWORD, IDENTIFIER=IDENTIFIER)
AMBIT = r'({NAMESPACE}::)*'.format(NAMESPACE=NAMESPACE)
TYPE = r'{AMBIT}{TYPENAME}'.format(AMBIT=AMBIT, TYPENAME=TYPENAME)
VARIABLE = r'{AMBIT}{VARNAME}'.format(AMBIT=AMBIT, VARNAME=VARNAME)
ACCESSOR = r'\.|-\>'
MEMBER = r'{VARIABLE}{ACCESSOR}{MEMBERNAME}'.format(
    VARIABLE=VARIABLE, ACCESSOR=ACCESSOR, MEMBERNAME=MEMBERNAME)
FUNCTION = VARIABLE
MEMBER_FUNCTION = r'{VARIABLE}{ACCESSOR}{MEMBERNAME}'.format(
    VARIABLE=VARIABLE, ACCESSOR=ACCESSOR, MEMBERNAME=MEMBERNAME)
ARGLIST = r'([^,]*(,[^,]*)*)?'
FUNCTION_CALL = r'({FUNCTION}|{MEMBER_FUNCTION})\s*\({ARGLIST}\s*\)'.format(
    FUNCTION=FUNCTION, MEMBER_FUNCTION=MEMBER_FUNCTION, ARGLIST=ARGLIST)
REFERENCE_SPECIFIER = r'&'
POINTER_SPECIFIER = r'\*'
SIZE_SPECIFIER = r'{POSITIVE}|{VARIABLE}|{MEMBER}|{FUNCTION_CALL}'.format(POSITIVE=POSITIVE,
    VARIABLE=VARIABLE, MEMBER=MEMBER, FUNCTION_CALL=FUNCTION_CALL)
ARRAY_SPECIFIER = r'\[\s*({SIZE_SPECIFIER})?\s*\]'.format(SIZE_SPECIFIER=SIZE_SPECIFIER)
VARNAME_PREFIX = r'({REFERENCE_SPECIFIER}|{POINTER_SPECIFIER})'.format(
    REFERENCE_SPECIFIER=REFERENCE_SPECIFIER, POINTER_SPECIFIER=POINTER_SPECIFIER)
VARNAME_SUFFIX = ARRAY_SPECIFIER
TYPE_MODIFIER = r'volatile|const|static|register|signed|unsigned|long|short'

MODIFIED_TYPE = r'(({TYPE_MODIFIER})\s+)*{TYPE}'.format(TYPE_MODIFIER=TYPE_MODIFIER, TYPE=TYPE)
G_MODIFIED_TYPE = r'(?P<modifiers>(({TYPE_MODIFIER})\s+)*)(?P<type>{TYPE})'.format(TYPE_MODIFIER=TYPE_MODIFIER, TYPE=TYPE)
reMODIFIED_TYPE = re.compile(G_MODIFIED_TYPE)

VALUE = r'{CONSTANT}|{VARIABLE}|{MEMBER}|{FUNCTION_CALL}'.format(
    CONSTANT=CONSTANT, VARIABLE=VARIABLE, MEMBER=MEMBER, FUNCTION_CALL=FUNCTION_CALL)

NEW_CAST = r'(dynamic_cast|const_cast|reinterpret_cast)\s*\<\s*{MODIFIED_TYPE}\s*(\*\s*)*\>\s*\(.*\)'.format(
    MODIFIED_TYPE=MODIFIED_TYPE)
OLD_CAST = r'\(\s*{MODIFIED_TYPE}\s*(\*\s*)*\)({VALUE})'.format(MODIFIED_TYPE=MODIFIED_TYPE, VALUE=VALUE)

CAST = r'({NEW_CAST})|({OLD_CAST})'.format(NEW_CAST=NEW_CAST, OLD_CAST=OLD_CAST)

RVALUE = r'{VALUE}|{CAST}|'.format(VALUE=VALUE, CAST=CAST)

VARINIT = '(({VARNAME_PREFIX}\s*)+|\s+)({VARNAME})\s*({VARNAME_SUFFIX})?(\s*=\s*({RVALUE}))?'.format(
    VARNAME_PREFIX=VARNAME_PREFIX, VARNAME=VARNAME, VARNAME_SUFFIX=VARNAME_SUFFIX, RVALUE=RVALUE)

G_VARINIT = '((?P<prefix>({VARNAME_PREFIX}\s*)+)|\s+)(?P<name>{VARNAME})\s*(?P<suffix>{VARNAME_SUFFIX})?(\s*=\s*({VALUE}))?'.format(
    VARNAME_PREFIX=VARNAME_PREFIX, VARNAME=VARNAME, VARNAME_SUFFIX=VARNAME_SUFFIX, VALUE=VALUE)
reVARINIT = re.compile(G_VARINIT)

VARDECL = r'(?P<modified_type>{MODIFIED_TYPE})\s*(?P<varlist>{VARINIT}(,{VARINIT})*)\s*;'.format(
    MODIFIED_TYPE=MODIFIED_TYPE, VARINIT=VARINIT)
reVARDECL = re.compile(VARDECL)


def gather_vars(code, types=None):
    """This method analizes the code and returns either false if something is wrong
    or a dictionnaire with the structure:
        varname:{type:(typename), ptr:(boolean),  const:(boolean), array:(boolean)}
    if types is specified, it must be a dicctionary typename:<type object>. This case,
    varname:type is translated to <type object> and ignored if not present
    """
    code = clean_code(code)
    if not code:
        return False
    _vars = {}
    for match in reVARDECL.finditer(code):
        modified_type = match.group('modified_type')
        varlist = match.group('varlist')
        match_modified_type = reMODIFIED_TYPE.match(modified_type)
        if not match_modified_type:
            print "Error: modified_type {modified_type} can't be analised\n".format(modified_type=modified_type)
            continue
        modifiers = match_modified_type.group('modifiers')
        _type = match_modified_type.group('type')
        if types:
            if _type not in types:
                continue
            _type = types[_type]
        if modifiers:
            _volatile = 'volatile' in modifiers
            _const = 'const' in modifiers
            _static = 'static' in modifiers
            _register = 'register' in modifiers
            if 'long' in modifiers:
                _type = 'long {}'.format(_type)
            if 'short' in modifiers:
                _type = 'short {}'.format(_type)
            if 'signed' in modifiers:
                _type = 'signed {}'.format(_type)
            if 'unsigned' in modifiers:
                _type = 'unsigned {}'.format(_type)
        else:
            _volatile = False
            _const = False
            _static = False
            _register = False
        count = 0
        for match_varinit in reVARINIT.finditer(varlist):
            count = count + 1
            _prefix = match_varinit.group('prefix')
            _name = match_varinit.group('name')
            _suffix = match_varinit.group('suffix')
            if len(_name) == 0:
                print "Error: name in {varinit} can't be extracted\n".format(varinit=match.expand(G_VARINIT))
                continue
            if _prefix:
                _ptr = '*' in _prefix
                _ref = '&' in _prefix
            else:
                _ptr = False
                _ref = False
            if _suffix:
                _array = '[' in _suffix
            else:
                _array = False
            _vars.update({_name: {
                'type': _type,
                'static': _static,
                'volatile': _volatile,
                'const': _const,
                'register': _register,
                'ptr': _ptr,
                'ref': _ref,
                'array': _array
                }})
        if count == 0:
            print "Error : failed to parse varlist {varlist}\n".format(varlist=varlist)
    return _vars




"""Subclass of NewGitRemote, which is generated by wxFormBuilder."""

import wx
import wxx
import app.resources as rc
from activity.git.ui import ui as ui


# Implementing NewGitRemote
class GitRemoteDialog(ui.NewGitRemote):
    """
    This dialog allows to create a new git remote.

    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Init"""
        self.container = container
        super(GitRemoteDialog, self).__init__(parent)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("git_remote"))
        self.SetIcon(icon)

    def Validate(self):
        """Validate dialog"""
        self._name = self.m_name.GetValue().strip()
        self._url = self.m_url.GetValue().strip()
        self._pass = self.m_password.GetValue()
        if not self._name:
            wx.MessageBox("Remote repository name is mandatory.", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['name'] = self._name
        kwargs['url'] = self._url
        kwargs['pass'] = self._pass
        kwargs['new'] = True  # This arg indicates that the remote must be added to the repo
        return kwargs

    # Handlers for NewGitRemote events.
    def OnCancel(self, event):
        """Handle cancel event"""
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Handle OnOk"""
        if self.Validate():
            self.EndModal(wx.ID_OK)




# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 23:24:30 2013

@author: mel
"""

import os
import model
import tran
from model.writer import writer
import app.resources as rc
from tran import TransactionStack
from Module import Module


class pyBoostModule(Module):
    """Implements a Module representation"""
    class_container = False
    folder_container = True
    diagram_container = True
    namespace_container = True
    function_container = True
    variable_container = True
    enum_container = True

    def __init__(self, **kwargs):
        """Initialization"""
        self._content = kwargs.get('content')
        self._uids = kwargs.get('uids')
        super(pyBoostModule, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {'content': self._content}
        kwargs.update(super(pyBoostModule, self).get_kwargs())
        return kwargs

    @tran.DelayedMethod
    def ExportCppCodeFiles(self, force=True):
        """Do code generation"""
        self.updateSources(force)

    def WriteCode(self, pf):
        """Write code for module"""
        pf.writeln(self._content)

    def updateSources(self, force=False):
        """does source generation"""
        sources_dir = self.project.sources_dir
        fname = os.path.realpath(os.path.join(sources_dir, self._source))
        if force or not os.path.exists(fname) or self._lastSrcTime is None:
            regenerate = True
        elif os.path.getmtime(fname) < self._lastSrcTime:
            regenerate = True
        project = self.project
        if regenerate:
            with open(fname, 'w') as f:
                pf = writer.for_file(f)
                pf.writeln("// {user.before.include.begin}")
                pf.writenl()
                pf.writeln("// {user.before.include.end}")
                pf.writenl()
                if project._useMaster:
                    pf.writeln('#include "{}"'.format(project._masterInclude))
                    pf.writenl()
                pf.writeln("// {user.before.code.begin}")
                pf.writenl()
                pf.writeln("// {user.before.code.end}")
                pf.writenl()
                self.WriteCode(pf)
                pf.writenl()
                pf.writeln("// {user.after.code.begin}")
                pf.writenl()
                pf.writeln("// {user.after.code.end}")
                pf.writenl()

    def updateHeaders(self, force=False):
        """Realiza la generacion de fuentes"""
        pass

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("pyboost")

    @property
    def inner_class(self):
        """Get the innermost class container"""
        return None

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Module, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            self.project.ExportCppCodeFiles(force=True)



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
import uuid
import app
from ctx import theContext as context
import model
import tran


class TComponent(model.TCommon, tran.TransactionObject):
    """We need to be able to deal with several kinds of TransactionObject.
    For example, with TransactionFSObject now and with TransactionDBObject in a near future.
    For this reason we need to split the data management and transaction management."""
    def __init__(self, **kwargs):
        """init"""
        super(TComponent, self).__init__(**kwargs)





from TransactionStack import copy_temps
from TransactionStack import respect_visuals_from
from TransactionStack import Transaction
from TransactionStack import UndoNew
from TransactionStack import RedoNew
from TransactionStack import UndoDelete
from TransactionStack import RedoDelete
from TransactionStack import UndoChange
from TransactionStack import RedoChange
from TransactionStack import TransactionStack
from TransactionObject import TransactionObject
from TransactionFSObject import TransactionFSObject
from TransactionalMethod import TransactionalMethod
from TransactionalMoveObject import TransactionalMoveObject
from DelayedMethod import DelayedMethod



"""Subclass of TasksView, which is generated by wxFormBuilder."""

import copy
import wx
import wxx
import tran
import deco
import app
import app.resources as rc
import model
import model.tasks as tasks
from ctx import localcontext as context
from app.ui.view import BaseView
from activity.tasks.ui import ui
from activity.tasks.ui import dlg as mdlg
import wx.propgrid as pg
import app.clipboard as clipboard


# base class for clipboard data
class clip_item_tasks(object):
    """clipboard format"""

    def __init__(self, object):
        """
        Initialize clipboard data.
        The object is the selected data
        """
        self.proj = object.project
        self.type = type(object)
        self.args = object.get_kwargs()


CLIP_FORMAT_TASKS = clipboard.register(clip_item_tasks)


# Implementing TasksView
class TasksView(BaseView, ui.TasksView):
    """Class for tasks view"""
    perspective = ''

    # add folder
    _addFolderId = wx.Window.NewControlId()
    _newTaskId = wx.Window.NewControlId()

    #update_context : these properties will be used for update menus
    @property
    def can_copy(self):
        """return info about the possibility to copy"""
        selected = self.selected
        if selected and type(self.selected) is tasks.Task:
            return True
        return False

    @property
    def can_paste(self):
        """is paste possible?"""
        selected = self.selected
        if not selected or clipboard.info != CLIP_FORMAT_TASKS:
            return False
        if not selected.inner_task_container:
            return False
        return True

    @property
    def can_edit_properties(self):
        """is possible to edit properties?"""
        if type(self.selected) is tasks.Tasks:
            return True
        return False

    @property
    def can_delete(self):
        """can delete selected object?"""
        if not self.selected:
            return False
        if type(self.selected) in [model.Project, model.Workspace]:
            return False
        return self.selected.can_delete

    @property
    def can_add_folder(self):
        """can be a folder addeded?"""
        if not self.selected:
            return False
        if type(self.selected) in [model.Project, model.Workspace]:
            return False
        return True

    def __init__(self, parent, root=None):
        """initialize view"""
        self.frame = context.frame
        self._infoHeight = -200
        super(TasksView, self).__init__(parent)
        self.imglist = rc.GetBitmapImageList()
        self.m_tree.SetImageList(self.imglist)
        self.m_tree.AddRoot('Tasks')
        #self.m_evtHandler = wx.PyEvtHandler()
        self._show_info = False
        self._create_menus()
        self._create_toolbars()
        self._bind_events()
        if root is None:
            theApp = context.app
            for wrk in theApp.workspaces:
                self.insert(wrk)
        # enhacement for popu subviews
        else:
            self.insert(root)

    def GetViewStatus(self):
        """Get working info about the view"""
        return str(self.m_tree.GetExpansionState())

    def SetViewStatus(self, status):
        """Set working info about the view"""
        try:
            l = eval(status)
            self.m_tree.SetExpansionState(l)
            return True
        except:
            return False

    def _create_menus(self):
        """Create a custom menu entry in main menu"""
        self._menu = wx.Menu()
        # add folder
        self._addFolder = wx.MenuItem(self._menu, self._addFolderId, u"Folder ...\tShift+Ctrl+F",
            u"add folder", wx.ITEM_NORMAL)
        self._addFolder.SetBitmap(wx.Bitmap(u"app/res/folder.xpm", wx.BITMAP_TYPE_ANY))
        self._menu.AppendItem(self._addFolder)

        # New task
        self._newTask = wx.MenuItem(self._menu, self._newTaskId, u"New task",
            u"create new task repository", wx.ITEM_NORMAL)
        self._newTask.SetBitmap(wx.Bitmap('app/res/new_task.xpm'))
        #self._newTask.SetBitmap(wx.ArtProvider.GetBitmap(wx.ART_NEW, wx.ART_MENU))
        self._menu.AppendItem(self._newTask)
        self.RegisterMenu('Tasks', self._menu)

    def _create_toolbars(self):
        """Create the associated toolbars"""
        # common elements
        self._auiToolBarElements = wx.aui.AuiToolBar(self.frame, wx.ID_ANY, wx.DefaultPosition,
            wx.DefaultSize, wx.aui.AUI_TB_GRIPPER | wx.aui.AUI_TB_HORZ_LAYOUT)
        els = [self._addFolder, self._newTask]
        bar = self._auiToolBarElements
        bar.SetMinSize(wx.Size(24, 24))
        for el in els:
            bar.AddTool(el.GetId(), el.GetLabel(), el.GetBitmap())
        bar.Realize()
        self.frame.m_mgr.AddPane(bar, wx.aui.AuiPaneInfo().Name(u"tasks_toolbar").Top().Hide().
            Caption(u"elements").PinButton(True).Gripper().Dock().Resizable().DockFixed(False).Row(1).Position(0).Layer(10).ToolbarPane())
        self.RegisterToolbar(bar)

    def _bind_events(self):
        """Connect handlers"""
        self.m_tree.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnTreeSelChanged)
        self.m_tree.Bind(wx.EVT_RIGHT_DOWN, self.OnTreeMenu)

        self.m_tree.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnEditProperties)

        self.m_tree.Bind(wx.EVT_TREE_BEGIN_DRAG, self.OnTasksBeginDrag)
        self.m_tree.Bind(wx.EVT_TREE_END_DRAG, self.OnTasksEndDrag)

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddFolder, id=self._addFolderId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddFolder, id=self._addFolder.GetId())

        self.BindSpecial(wx.EVT_UPDATE_UI, self.OnUpdateAddNewTask, id=self._newTaskId)
        self.BindSpecial(wx.EVT_MENU, self.OnAddNewTask, id=self._newTask.GetId())

        super(TasksView, self)._bind_events()

    def insert(self, element):
        """Nested insert elements in tree"""
        if self.DoRenderAddElement(element):
            for cls in element._child:
                for k in element[cls]:
                    self.insert(k)

    def DoRenderAddElement(self, obj):
        """Inserts element in tree"""
        # model tree
        treeOrder = [model.Workspace, model.Project, tasks.PendingTasks,
            tasks.CurrentTasks, tasks.FinishedTasks, tasks.TaskFolder, tasks.Task]
        if type(obj) not in treeOrder:
            return False
        ti = treeOrder.index(type(obj))
        #check parent
        if self.m_tree.HoldsObject(obj.parent):
            p = obj.parent
        else:
            p = self.m_tree.GetRootItem()
        if obj.parent is not None:
            # find some major friend item with the same class
            tribal = obj.parent[type(obj)]
            index = tribal.index(obj)
            pre = None
            while index > 0 and pre is None:
                index = index - 1
                candidate = tribal[index]
                if not self.m_tree.HoldsObject(candidate):
                    continue
                pre = candidate
            if pre is not None:
                self.m_tree.InsertItem(p, pre, obj.label,
                    obj.bitmap_index, obj.bitmap_index, obj)
                if hasattr(obj, 'bitmap_open_index'):
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
                return True
        itemCount = 0
        citem, cookie = self.m_tree.GetFirstChild(p)
        if type(citem) is wx.TreeItemId:
            citem = self.m_tree.__fer__(citem)
        if type(citem) in treeOrder:
            if ti <= treeOrder.index(type(citem)):
                self.m_tree.PrependItem(p, obj.label,
                     obj.bitmap_index, obj.bitmap_index, obj)
                if hasattr(obj, 'bitmap_open_index'):
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
                return True
        while type(citem) is not wx.TreeItemId or citem.IsOk():
            itemCount = itemCount + 1
            citem, cookie = self.m_tree.GetNextChild(p, cookie)
            if type(citem) not in treeOrder:
                continue
            if ti <= treeOrder.index(type(citem)):
                self.m_tree.InsertItemBefore(p, itemCount,
                     obj.label,
                     obj.bitmap_index, obj.bitmap_index, data=obj)
                if hasattr(obj, 'bitmap_open_index'):
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
                    self.m_tree.SetItemImage(obj,
                        obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
                return True
        #Ok, do apppend
        self.m_tree.AppendItem(p, obj.label,
            obj.bitmap_index, obj.bitmap_index, obj)
        if hasattr(obj, 'bitmap_open_index'):
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
        return True

    def DoRenderRemoveElement(self, obj):
        """Do remove element in tree"""
        if self.m_tree.HoldsObject(obj):
            self.m_tree.Delete(obj)

    def OnUpdateEditProperties(self, event):
        """Do update edit properties"""
        event.Enable(self.can_edit_properties)

    def OnEditProperties(self, event):
        """Edit the task"""
        if type(self.selected) is model.tasks.TaskFolder:
            self.OnEditTaskFolderProperties(self.selected)
        else:
            self.OnEditTaskProperties(self.selected)
        return (context.frame, self.selected)

    @tran.TransactionalMethod('edit task {}')
    @wxx.EditionDialog(mdlg.TaskDialog)
    def OnEditTaskProperties(self, obj):
        """Edit the task"""
        return (context.frame, obj)

    @tran.TransactionalMethod('edit task folder {}')
    @wxx.EditionDialog(mdlg.TaskFolderDialog)
    def OnEditTaskFolderProperties(self, obj):
        """Edit the task folder"""
        return (context.frame, obj)

    def OnTasksBeginDrag(self, event):
        '''Allow drag-and-drop for leaf nodes.'''
        obj = self.m_tree.__fer__(event.GetItem())
        if obj is None:
            return
        if obj.draggable():
            event.Allow()
            self.dragObject = obj

    def OnTasksEndDrag(self, event):
        '''Handle drop operation'''
        if not self.dragObject:
            return
        obj = self.m_tree.__fer__(event.GetItem())
        if obj is None or self.dragObject in obj.path:
            return
        self.dragObject.drop(obj)

    def UpdateElement(self, obj):
        """Update the tree label for a object"""
        if not self.m_tree.HoldsObject(obj):
            return
        self.m_tree.SetItemText(obj, obj.label)
        self.m_tree.SetItemImage(obj,
            obj.bitmap_index, wx.TreeItemIcon_Normal)
        if hasattr(obj, 'bitmap_open_index'):
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_Expanded)
            self.m_tree.SetItemImage(obj,
                obj.bitmap_open_index, wx.TreeItemIcon_SelectedExpanded)
        self.m_tree.SetItemImage(obj,
            obj.bitmap_index, wx.TreeItemIcon_Selected)

    def OnUpdateDelete(self, event):
        """Update delete"""
        event.Enable(self.can_delete)

    def OnTreeSelChanged(self, event):
        """Handle select changed"""
        self.selected = self.m_tree.GetSelection()
        if self.selected and type(self.selected) is tasks.Task:
            if not self._show_info:
                self.m_info = ui.Info(self.m_splitter)
                self.m_splitter.SplitHorizontally(self.m_treePane, self.m_info, self._infoHeight)
                self._show_info = True
                # after show the info, scroll the tree to ensure the item is visible
                wx.YieldIfNeeded()
                self.m_tree.EnsureVisible(self.selected)
            else:
                self.m_info.m_propertyGrid.Clear()
            # fill the grid with the relevant information
            self.m_info.m_text.SetValue(self.selected.note)
            prop = pg.NewStringProperty(label='status', value=self.selected._status)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('status')
            prop = pg.NewStringProperty(label='priority', value=self.selected._priority)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('priority')
            prop = pg.NewStringProperty(label='type', value=self.selected._taskType)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('type')
            prop = pg.NewStringProperty(label='created', value=self.selected._dateCreated)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('created')
            prop = pg.NewStringProperty(label='started', value=self.selected._dateBegin)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('started')
            prop = pg.NewStringProperty(label='finished', value=self.selected._dateEnd)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('finished')
            prop = pg.NewStringProperty(label='reference', value=self.selected._reference)
            self.m_info.m_propertyGrid.Append(prop)
            self.m_info.m_propertyGrid.SetPropertyReadOnly('reference')
        elif self._show_info:
            self._infoHeight = self.m_splitter.GetSashPosition()
            self.m_splitter.Unsplit(self.m_info)
            self.m_info.Destroy()
            self.m_info = None
            self._show_info = False

    def OnTreeMenu(self, event):
        """Handles context tree popup menu"""
        (item, where) = self.m_tree.HitTest(event.GetPosition())
        if item is None:
            return
        self.m_tree.SelectItem(item)
        obj = self.selected
        if obj is None:
            return
        menu = wx.Menu()
        # add elements filtering disabled
        res = app.ui.clone_mnu(self._menu, parent=menu, enabled=True)
        if res is not None:
            menu.AppendSeparator()
            app.ui.append_menuitem_copy(menu, self.frame.editProperties)
            self.PopupMenu(menu, event.GetPosition())
        menu.Destroy()

    def OnDelete(self, event):
        """delete element"""
        obj = self.selected
        if type(obj) is model.tasks.TaskFolder:
            tran.TransactionStack.DoBeginTransaction("delete task folder {}".format(obj._name))
        else:
            tran.TransactionStack.DoBeginTransaction("delete task {}".format(obj._name))
        obj.Delete()
        tran.TransactionStack.DoCommit()

    def OnUpdateAddFolder(self, event):
        """Update add folder method"""
        event.Enable(self.can_add_folder)

    def OnUpdateAddNewTask(self, event):
        """Update add task"""
        event.Enable(self.can_add_task)

    @property
    def can_add_task(self):
        """a task may be added?"""
        if not self.selected:
            return False
        if type(self.selected) in [model.Project, model.Workspace]:
            return False
        return True

    def OnUpdateSaveProject(self, event):
        """Handle update save project"""
        event.Enable(bool(self.selected and self.selected.project))

    def OnSaveProject(self, event):
        """Handle save project"""
        obj = self.selected
        project = obj.project
        if project is None:
            return
        context.app.SaveProject(project)

    @tran.TransactionalMethod('add new task {}')
    @wxx.CreationDialog(mdlg.TaskDialog, model.tasks.Task)
    def OnAddNewTask(self, event):
        """Add task"""
        return (context.frame, self.selected)

    @tran.TransactionalMethod('add task folder {}')
    @wxx.CreationDialog(mdlg.TaskFolderDialog, model.tasks.TaskFolder)
    def OnAddFolder(self, event):
        """Handle add folder command"""
        return (context.frame, self.selected)

    def OnUpdateEditCopy(self, event):
        """Handle update edit copy"""
        event.Enable(self.can_copy)

    def OnEditCopy(self, event):
        """Handle edit copy event"""
        self.copyType = type(self.selected)
        self.copyArgs = self.selected.get_kwargs()

    def OnUpdateEditCut(self, event):
        """Handle cut event"""
        event.Enable(self.can_copy and self.can_delete)

    @tran.TransactionalMethod('cut task')
    def OnEditCut(self, event):
        """Handle cut event"""
        self.copyType = type(self.selected)
        self.copyArgs = self.selected.get_kwargs()
        self.OnDelete(event)

    def OnUpdateEditPaste(self, event):
        """Update edit paste"""
        event.Enable(self.can_paste)

    @tran.TransactionalMethod('paste task')
    def OnEditPaste(self, event):
        """Handle paste event"""
        """Handle paste argument"""
        kwargs = copy.copy(self.copyArgs)
        kwargs['parent'] = self.selected.inner_task_container
        model.tasks.Task(**kwargs)
        return True

    @deco.classproperty
    def name(cls):
        """returns the name of this view"""
        return 'Tasks'

    @classmethod
    def bitmap_index(cls):
        """return the bitmap index"""
        return rc.GetBitmapIndex('tasks')

    @classmethod
    def bitmap(cls):
        """return the bitmap"""
        return rc.GetBitmap('tasks')

    def TreeLeftKey(self, event):
        """If the selected node is expanded, simply collapse it.
        If not, navigate through parent"""
        if not self.selected:
            return
        if self.m_tree.IsExpanded(self.selected):
            self.m_tree.Collapse(self.selected)
        else:
            parent = self.selected.parent
            if self.m_tree.HoldsObject(parent):
                self.m_tree.SelectItem(parent)





# -*- coding: utf-8 -*-
import wx


def first(*args):
    """return first not None argument or None"""
    for arg in args:
        if not arg is None:
            return arg
    return None

def settuple(t, index, value):
    l = list(t)
    l[index] = value
    return tuple(l)


class TreeCtrl(wx.TreeCtrl):
    """Special override of wxTreeCtrl providing TCommon oriented features.
    The idea beyond this specialization is to provide transparent integration
    of TCommon elements in the tree, hidding its characteristics."""
    def __init__(self, *args, **kwargs):
        """ __init__(self, Window parent, int id=-1, Point pos=DefaultPosition,
        Size size=DefaultSize, long style=TR_DEFAULT_STYLE,
        Validator validator=DefaultValidator,
         String name=TreeCtrlNameStr) -> TreeCtrl"""
        self._map = {}
        super(TreeCtrl, self).__init__(*args, **kwargs)


    def GetItemChildren(self, item=None, recursively=False):
        """ Return the children of item as a list. """
        if not item:
            item = self.GetRootItem()
            if not item:
                return []
        children = []
        child, cookie = self.GetFirstChild(item)
        while child:
            children.append(child)
            if recursively:
                children.extend(self.GetItemChildren(child, True))
            child, cookie = self.GetNextChild(item, cookie)
        return children

    def GetExpansionState(self):
        """ GetExpansionState() -> list of expanded items. Expanded items
        are coded as determined by the result of GetItemIdentity(item). """
        root = self.GetRootItem()
        if not root:
            return []
        if self.HasFlag(wx.TR_HIDE_ROOT):
            return self.GetExpansionStateOfChildren(root)
        else:
            return self.GetExpansionStateOfItem(root)

    def GetExpansionStateOfItem(self, item):
        """return a diccionary about expansion status"""
        if self.IsExpanded(item):
            return {'self': True, 'child': self.GetExpansionStateOfChildren(item)}
        else:
            return {'self': False}

    def GetExpansionStateOfChildren(self, item):
        """return an array with expansion status"""
        return [self.GetExpansionStateOfItem(child)
            for child in self.GetItemChildren(item)]

    def SetExpansionState(self, listOfExpandedItems):
        """ SetExpansionState(listOfExpandedItems). Expands all tree items
        whose identity, as determined by GetItemIdentity(item), is present
        in the list and collapses all other tree items. """
        root = self.GetRootItem()
        if not root:
            return
        if self.HasFlag(wx.TR_HIDE_ROOT):
            self.SetExpansionStateOfChildren(listOfExpandedItems, root)
        else:
            self.SetExpansionStateOfItem(listOfExpandedItems, root)

    def SetExpansionStateOfItem(self, listOfExpandedItems, item):
        """Recovers expansion status (temptative)"""
        if listOfExpandedItems['self']:
            self.Expand(item)
            self.SetExpansionStateOfChildren(listOfExpandedItems['child'], item)
        else:
            self.Collapse(item)

    def SetExpansionStateOfChildren(self, listOfExpandedItems, item):
        """Recovers expansion status (temptative)"""
        i = 0
        for child in self.GetItemChildren(item):
            if i > len(listOfExpandedItems):
                break
            self.SetExpansionStateOfItem(listOfExpandedItems[i], child)
            i = i + 1

    def __getargchoice__(self, argpos, name1, name2, sync, *args, **kwargs):
        """
        Follow the correct interpretation for some argument that must be
        eiter a treeid or a object, this method returns the pair (id, obj).
        The argpos is the postion of the argument when received in positional
        args, and name1, name2 are the optional names when it's a mapped kwarg.
        The sync argument do object matching, if possible, in dictionary.
        """
        if len(args) > argpos:
            unk = args[argpos]
        else:
            unk = kwargs.get(name1, kwargs.get(name2, None))
        if unk is None:
            return None, None
        if isinstance(unk, wx.TreeItemId):
            item = unk
            obj = None
            if sync:
                data = super(TreeCtrl, self).GetItemData(item)
                if data is not None:
                    obj = data.GetData()
        else:
            obj = unk
            item = self.__ref__(obj)
        return obj, item

    def __fer__(self, *args, **kwargs):
        """__fer__(self, TreeItemId item) -> object.
        Finds the object and returns it or None"""
        if len(args) > 0:
            item = args[0]
        else:
            item = kwargs.get('item', None)
        if item is None or not item.IsOk():
            return None
        data = super(TreeCtrl, self).GetItemData(item)
        if data is not None:
            obj = data.GetData()
            if self.__ref__(obj) == item:
                return obj
        return None

    def __ref__(self, *args, **kwargs):
        """Gets the tree reference for any object"""
        import model
        if len(args) > 0:
            obj = args[0]
        else:
            obj = kwargs.get('obj', None)
        if obj is not None:
            if isinstance(obj, model.TCommon):
                return self._map.get(obj._uid, None)
            else:
                return self._map.get(obj, None)
        return None

    def __addref__(self, obj, treeid):
        """Add a map inside """
        import model
        if isinstance(obj, model.TCommon):
            self._map[obj._uid] = treeid
        else:
            self._map[obj] = treeid

    def __subref__(self, obj):
        """Remove a object reference"""
        import model
        if isinstance(obj, model.TCommon):
            self._map.pop(obj._uid, False)
        else:
            self._map.pop(obj, False)

    def HoldsObject(self, obj):
        """HoldsObject(self, TCommon obj) -> boolean"""
        import model
        if obj is None:
            return False
        if isinstance(obj, model.TCommon):
            return obj._uid in self._map
        else:
            return obj in self._map

    def AddRoot(self, *args, **kwargs):
        """AddRoot(self, String text, int image=-1, int selectedImage=-1,
            {TreeItemData data=None|TCommon obj=None}) -> {TreeItemId |TCommon}"""
        if len(args) >= 1:
            nargs = ['image', 'selectedImage', 'data']
            for i in range(len(args) - 1):
                kwargs[nargs[i]] = args[i + 1]
            args = (args[0],)
        else:
            args = ('')
        unk = kwargs.get('data', kwargs.get('obj', None))
        obj = None
        data = None
        if unk is not None:
            if isinstance(unk, wx.TreeItemData):
                data = unk
            else:
                obj = unk
                data = wx.TreeItemData(obj)
        kwargs['data'] = data
        treeid = super(TreeCtrl, self).AddRoot(*args, **kwargs)
        if obj is None:
            return treeid
        self.__addref__(obj, treeid)
        return obj

    def AppendItem(self, *args, **kwargs):
        """AppendItem(self, {TreeItemId|TCommon} parent, String text,
        int image=-1, int selectedImage=-1,
        {TreeItemData data=None|TCommon obj=None}) -> TreeItemId"""
        pobj, parent = self.__getargchoice__(0, 'parent', 'parent', True,
            *args, **kwargs)
        if len(args) >= 2:
            nargs = ['image', 'selectedImage', 'data']
            for i in range(len(args) - 2):
                kwargs[nargs[i]] = args[i + 2]
            args = (parent, args[1])
        else:
            args = (parent, '')
        unk = kwargs.get('data', kwargs.get('obj', None))
        obj = None
        data = None
        if unk is not None:
            if isinstance(unk, wx.TreeItemData):
                data = unk
            else:
                obj = unk
                data = wx.TreeItemData(obj)
        kwargs['data'] = data
        treeid = super(TreeCtrl, self).AppendItem(*args, **kwargs)
        if obj is None:
            return treeid
        self.__addref__(obj, treeid)
        return obj

    def Collapse(self, *args, **kwargs):
        """Collapse(self, {TreeItemId item, TCommon obj})"""
        # we allow to collapse over obj
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).Collapse(item)

    def CollapseAllChildren(self, *args, **kwargs):
        """CollapseAllChildren(self, {TreeItemId item|TCommon obj})"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).CollapseAllChildren(item)

    def CollapseAndReset(self, *args, **kwargs):
        """CollapseAndReset(self, {TreeItemId item|TCommon obj})"""
        # This method is specially hard: we must travel over the
        # childrens for ensure unmap deleted childs
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).Collapse(item)
        while super(TreeCtrl, self).GetChildrenCount(item) > 0:
            child = super(TreeCtrl, self).GetFirstChild(item)
            self.Delete(child)
        self.__subref__(obj)

    def Delete(self, *args, **kwargs):
        """Delete(self, {TreeItemId item|TCommon obj})"""
        # we do it carefully. It's possible to use item instead of using
        # obj, so we must check the tree item data
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args,
            **kwargs)
        # delete childs first
        while super(TreeCtrl, self).GetChildrenCount(item) > 0:
            child = super(TreeCtrl, self).GetFirstChild(item)
            self.Delete(child)
        super(TreeCtrl, self).Delete(item)
        self.__subref__(obj)

    def DeleteAllItems(self, *args, **kwargs):
        """DeleteAllItems(self)"""
        self._map = {}
        super(TreeCtrl, self).DeleteAllItems()

    def DeleteChildren(self, *args, **kwargs):
        """DeleteChildren(self, {TreeItemId item|TCommon obj})"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        while super(TreeCtrl, self).GetChildrenCount(item) > 0:
            child = super(TreeCtrl, self).GetFirstChild(item)
            self.Delete(child)

    def EditLabel(self, *args, **kwargs):
        """EditLabel(self, {wxTreeItemId item|TCommon obj})"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).EditLabel(item)

    def EnsureVisible(self, *args, **kwargs):
        """EnsureVisible(self, {wxTreeItemId item|TCommon obj})"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).EnsureVisible(item)

    def Expand(self, *args, **kwargs):
        """Expand(self, {wxTreeItemId item|TCommon obj})"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).Expand(item)

    def ExpandAllChildren(self, *args, **kwargs):
        """ExpandAllChildren(self, {wxTreeItemId item|TCommon obj})"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).ExpandAllChildren(item)

    def GetBoundingRect(self, *args, **kwargs):
        """GetBoundingRect(self, {wxTreeItemId item|TCommon obj},
            bool textOnly=False) -> PyObject"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        return super(TreeCtrl, self).GetBoundingRect(*args, **kwargs)

    def GetChildrenCount(self, *args, **kwargs):
        """GetChildrenCount(self, {TreeItemId item|TCommon obj},
        bool recursively=True) -> size_t"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        return super(TreeCtrl, self).GetChildrenCount(*args, **kwargs)

    def GetFirstChild(self, *args, **kwargs):
        """GetFirstChild(self, {TreeItemId item|TCommon obj}) -> PyObject"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', False, *args, **kwargs)

        # the return in the original is a pair (treeid, cookie). We
        # will change this meaning for (obj, cookie) only when the
        # following conditions meets:
        # - the reference argument is an object rather than a item.
        # - the child has, itself an associated object
        # in other case, old result style will be used
        if obj is None:
            return super(TreeCtrl, self).GetFirstChild(item)
        citem, cookie = super(TreeCtrl, self).GetFirstChild(item)
        return first(self.__fer__(citem), citem), cookie

    def GetFirstVisibleItem(self, *args, **kwargs):
        """GetFirstVisibleItem(self) -> {TreeItemId|TCommon}"""
        item = super(TreeCtrl, self).GetFirstVisibleItem()
        return first(self.__fer__(item), item)

    def GetFocusedItem(self, *args, **kwargs):
        """GetFocusedItem(self) -> {TreeItemId|TCommon}"""
        item = super(TreeCtrl, self).GetFocusedItem()
        return first(self.__fer__(item), item)

    def GetItemBackgroundColour(self, *args, **kwargs):
        """GetItemBackgroundColour(self,
        {TreeItemId item|TCommon obj}) -> Colour"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).GetItemBackgroundColour(item)

    def GetItemFont(self, *args, **kwargs):
        """GetItemFont(self, {TreeItemId item|TCommon obj}) -> Font"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).GetItemFont(item)

    def GetItemImage(self, *args, **kwargs):
        """GetItemFont(self, {TreeItemId item|TCommon obj},
        int which=TreeItemIcon_Normal) -> Font"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        return super(TreeCtrl, self).GetItemImage(*args, **kwargs)

    def GetItemParent(self, *args, **kwargs):
        """GetItemParent(self, {TreeItemId item|TCommon obj})
        -> {TreeItemId|TCommon}"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if obj is None:
            return super(TreeCtrl, self).GetItemParent(item)
        #Ok, now, we attempt to extract parent object first
        pitem = super(TreeCtrl, self).GetItemParent(item)
        if not pitem.IsOk():
            return None
        return first(self.__fer__(pitem), pitem)

    def GetItemPyData(self, *args, **kwargs):
        """GetItemPyData(self, {TreeItemId item|TCommon obj) -> PyObject"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).GetItemPyData(item)

    def GetItemState(self, *args, **kwargs):
        """GetItemState(self, {TreeItemId item|TCommon obj}) -> int"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).GetItemState(item)

    def GetItemText(self, *args, **kwargs):
        """GetItemText(self, {TreeItemId item|TCommon obj}) -> int"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).GetItemText(item)

    def GetItemTextColour(self, *args, **kwargs):
        """GetItemText(self, {TreeItemId item|TCommon obj}) -> int"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).GetItemTextColour(item)

    def GetLastChild(self, *args, **kwargs):
        """GetLastChild(self, {TreeItemId item|TCommon obj})
        -> {TreeItemId|TCommon}"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, sync=False, **kwargs)
        if obj is None:
            return super(TreeCtrl, self).GetLastChild(item)
        citem = super(TreeCtrl, self).GetLastChild(item)
        return first(self.__fer__(citem), citem)

    def GetNextChild(self, *args, **kwargs):
        """GetNextChild(self, {TreeItemId item|TCommon obj},
        void cookie) -> PyObject"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        if obj is None:
            return super(TreeCtrl, self).GetNextChild(*args, **kwargs)
        citem, cookie = super(TreeCtrl, self).GetNextChild(
            *args, **kwargs)
        return first(self.__fer__(citem), citem), cookie

    def GetNextSibling(self, *args, **kwargs):
        """GetNextSibling(self, {TreeItemId item|TCommon obj})
            -> {TreeItemId|TCommon}"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, sync=False, **kwargs)
        citem = super(TreeCtrl, self).GetNextSibling(item)
        if obj is None:
            return citem
        return first(self.__fer__(citem), citem)

    def GetNextVisible(self, *args, **kwargs):
        """GetNextVisible(self, {TreeItemId item|TCommon obj})
        -> {TreeItemId|TCommon}"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, sync=False, **kwargs)
        citem = super(TreeCtrl, self).GetNextVisible(item)
        if obj is None:
            return citem
        return first(self.__fer__(citem), citem)

    def GetPrevSibling(self, *args, **kwargs):
        """GetPrevSibling(self, {TreeItemId item|TCommon obj})
            -> {TreeItemId|TCommon}"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, sync=False, **kwargs)
        citem = super(TreeCtrl, self).GetPrevSibling(item)
        if obj is None:
            return citem
        return first(self.__fer__(citem), citem)

    def GetPrevVisible(self, *args, **kwargs):
        """GetPrevVisible(self, {TreeItemId item|TCommon obj})
        -> {TreeItemId|TCommon}"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, sync=False, **kwargs)
        citem = super(TreeCtrl, self).GetPrevVisible(item)
        if obj is None:
            return citem
        return first(self.__fer__(citem), citem)

    def GetRootItem(self):
        """GetRootItem(self) -> {TreeItemId|TCommon}"""
        item = super(TreeCtrl, self).GetRootItem()
        return first(self.__fer__(item), item)

    def GetSelection(self):
        """GetSelection(self)-> {TreeItemId|TCommon}"""
        item = super(TreeCtrl, self).GetSelection()
        return first(self.__fer__(item), item)

    def HitTest(self, *args, **kwargs):
        """HitTest(Point point) -> {item, where)"""
        item, where = super(TreeCtrl, self).HitTest(*args, **kwargs)
        return first(self.__fer__(item), item), where

    def InsertItem(self, *args, **kwargs):
        """InsertItem(self, {TreeItemId|TCommon} parent,
            {TreeItemId idPrevious|TCommon objPrevious}, String text,
            int image=-1, int selectedImage=-1,
            {TreeItemData data|TComponen obj}= None)->{TreeItemId|TCommon}"""
        pobj, pitem = self.__getargchoice__(0, 'parent',
            'parent', True, *args, **kwargs)
        vobj, vitem = self.__getargchoice__(1, 'idPrevious',
            'objPrevious', True, *args, **kwargs)
        if len(args) >= 3:
            nargs = ['image', 'selectedImage', 'data']
            for i in range(len(args) - 3):
                kwargs[nargs[i]] = args[i + 3]
            args = (pitem, vitem, args[2])
        else:
            args = (pitem, vitem, '')
        unk = kwargs.get('data', kwargs.get('obj', None))
        obj = None
        data = None
        if unk is not None:
            if isinstance(unk, wx.TreeItemData):
                data = unk
            else:
                obj = unk
                data = wx.TreeItemData(obj)
        kwargs['data'] = data
        treeid = super(TreeCtrl, self).InsertItem(*args, **kwargs)
        if obj is None:
            return treeid
        self.__addref__(obj, treeid)
        return obj

    def InsertItemBefore(self, *args, **kwargs):
        """InsertItemBefore(self, {TreeItemId|TCommon} parent, size_t index, String text,
            int image=-1, int selectedImage=-1, {TreeItemData data|TComponen obj}= None)
            ->{TreeItemId|TCommon}"""
        pobj, pitem = self.__getargchoice__(0, 'parent', 'parent', True, *args, **kwargs)
        if len(args) >= 3:
            nargs = ['image', 'selectedImage', 'data']
            for i in range(len(args) - 3):
                kwargs[nargs[i]] = args[i + 3]
            args = (pitem, args[1], args[2])
        elif len(args) > 1:
            args = (pitem, args[1], 0)
        else:
            args = (pitem, '', 0)
        unk = kwargs.get('data', kwargs.get('obj', None))
        obj = None
        data = None
        if unk is not None:
            if isinstance(unk, wx.TreeItemData):
                data = unk
            else:
                obj = unk
                data = wx.TreeItemData(obj)
        kwargs['data'] = data
        item = super(TreeCtrl, self).InsertItemBefore(*args, **kwargs)
        if obj is None:
            return item
        self.__addref__(obj, item)
        return obj

    def IsBold(self, *args, **kwargs):
        """IsBold(self, {TreeItemId item|TCommon obj}) -> bool"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).IsBold(item)

    def IsEmpty(self, *args, **kwargs):
        """IsEmpty(self, {TreeItemId item|TCommon obj}) -> bool"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).IsEmpty(item)

    def IsExpanded(self, *args, **kwargs):
        """IsEmpty(self, {TreeItemId item|TCommon obj}) -> bool"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).IsExpanded(item)

    def IsSelected(self, *args, **kwargs):
        """IsSelected(self, {TreeItemId item|TCommon obj}) -> bool"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).IsSelected(item)

    def IsVisible(self, *args, **kwargs):
        """IsVisible(self, {TreeItemId item|TCommon obj}) -> bool"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).IsVisible(item)

    def ItemHasChildren(self, *args, **kwargs):
        """ItemHasChildren(self, {TreeItemId item|TCommon obj}) -> bool"""
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        return super(TreeCtrl, self).ItemHasChildren(item)

    def PrependItem(self, *args, **kwargs):
        """PrependItem(self, {TreeItemId|TCommon} parent, String text,
        int image=-1, int selectedImage=-1,
        {TreeItemData data=None|TCommon obj=None}) -> TreeItemId"""
        pobj, pitem = self.__getargchoice__(0, 'parent', 'parent', True, *args, **kwargs)
        if len(args) >= 2:
            nargs = ['image', 'selectedImage', 'data']
            for i in range(len(args) - 2):
                kwargs[nargs[i]] = args[i + 2]
            args = (pitem, args[1])
        else:
            args = (pitem, '')
        unk = kwargs.get('data', kwargs.get('obj', None))
        obj = None
        data = None
        if unk is not None:
            if isinstance(unk, wx.TreeItemData):
                data = unk
            else:
                obj = unk
                data = wx.TreeItemData(obj)
        kwargs['data'] = data
        item = super(TreeCtrl, self).PrependItem(*args, **kwargs)
        if obj is None:
            return item
        self.__addref__(obj, item)
        return obj

    def ScrollTo(self, *args, **kwargs):
        """ScrollTo(self, {TreeItemId item|TCommon obj}) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).ScrollTo(item)

    def SelectChildren(self, *args, **kwargs):
        """SelectChildren(self, {TreeItemId parent|TCommon obj}) """
        obj, parent = self.__getargchoice__(0, 'parent', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, parent)
        else:
            kwargs['parent'] = parent
        super(TreeCtrl, self).SelectChildren(*args, **kwargs)

    def SelectItem(self, *args, **kwargs):
        """SelectItem(self, {TreeItemId item|TCommon obj}, select=True) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SelectItem(*args, **kwargs)

    def SetFocusedItem(self, *args, **kwargs):
        """SetFocusedItem(self, {TreeItemId item|TCommon obj}) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).SetFocusedItem(item)

    def SetItemBackgroundColour(self, *args, **kwargs):
        """SetItemBackgroundColour(self, {TreeItemId item|TCommon obj}, Colour col) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemBackgroundColour(*args, **kwargs)

    def SetItemBold(self, *args, **kwargs):
        """SetItemBold(self, {TreeItemId item|TCommon obj}, bool bold=True) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 1:
            kwargs['bold'] = bool(args[1])
        args = (item, )
        super(TreeCtrl, self).SetItemBold(*args, **kwargs)

    def SetItemDropHighlight(self, *args, **kwargs):
        """SetItemDropHighlight(self, {TreeItemId item|TCommon obj}, bool highlight=True) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemDropHighlight(*args, **kwargs)

    def SetItemFont(self, *args, **kwargs):
        """SetItemFont(self, {TreeItemId item|TCommon obj}, Font font) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemFont(*args, **kwargs)

    def SetItemHasChildren(self, *args, **kwargs):
        """SetItemHasChildren(self, {TreeItemId item|TCommon obj}, bool has=True) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemHasChildren(*args, **kwargs)

    def SetItemImage(self, *args, **kwargs):
        """SetItemImage(self, {TreeItemId item|TCommon obj}, int wich=TreeItemIcon_Normal) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemImage(*args, **kwargs)

    def SetItemState(self, *args, **kwargs):
        """SetItemState(self, {TreeItemId item|TCommon obj}, int state) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemState(*args, **kwargs)

    def SetItemText(self, *args, **kwargs):
        """SetItemState(self, {TreeItemId item|TCommon obj}, String text) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemText(*args, **kwargs)

    def SetItemTextColour(self, *args, **kwargs):
        """SetItemTextColour(self, {TreeItemId item|TCommon obj}, Colour col) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        if len(args) > 0:
            args = settuple(args, 0, item)
        else:
            kwargs['item'] = item
        super(TreeCtrl, self).SetItemTextColour(*args, **kwargs)

    def SortChildren(self, *args, **kwargs):
        """SortChildren(self, {TreeItemId item|TCommon obj}) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).SortChildren(item)

    def Toggle(self, *args, **kwargs):
        """Toggle(self, {TreeItemId item|TCommon obj}) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).Toggle(item)

    def ToggleItemSelection(self, *args, **kwargs):
        """ToggleItemSelection(self, {TreeItemId item|TCommon obj}) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).ToggleItemSelection(item)

    def UnselectItem(self, *args, **kwargs):
        """UnselectItem(self, {TreeItemId item|TCommon obj}) """
        obj, item = self.__getargchoice__(0, 'item', 'obj', True, *args, **kwargs)
        super(TreeCtrl, self).UnselectItem(item)



"""Subclass of NewPyPackage, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


class PyPackageDialog(ui.NewPyPackage):
    """
    This dialog allows to create or edit
    a python package.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """initialice dialog"""
        self.container = container
        super(PyPackageDialog, self).__init__(parent)
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_package"))
        self.SetIcon(icon)

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['parent'] = self.container
        kwargs['name'] = self._name
        kwargs['note'] = self._note
        return kwargs

    def CopyAttributes(self, package):
        """Copy attributes to method"""
        package._name = self._name
        package.note = self._note

    def SetAttributes(self, package):
        """Setup attributes for editing already method"""
        self.m_comboBox5.SetValue(package.name)
        self.m_richText3.SetValue(package.note)
        self.SetTitle("Edit package")

    def Validate(self):
        """Validation"""
        self._name = self.m_package_name.GetValue().strip()
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("package name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._note = self.m_richText3.GetValue()
        return True

    def OnCancel(self, event):
        """Handle cancel dialog"""
        self.EndModal(wx.ID_CANCEL)

    # Handlers for NewPyPackage events.
    def OnOK(self, event):
        """Handle ok method"""
        if self.Validate():
            self.EndModal(wx.ID_OK)



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-

import wx.lib.agw.supertooltip as ttip


class SetInfo(object):
    """Class decorator for setting tooltip info into dialog"""
    def __init__(self, text, about="what's this?"):
        """"""
        self._text = text
        self._about = about

    def __call__(self, method):
        """"""
        def wrapped_call(*args, **kwargs):
            """Code dialog creation"""
            method(*args, **kwargs)
            this = args[0]
            tip = ttip.SuperToolTip(self._text)
            tip.SetHeader(self._about)
            tip.SetTarget(this.m_info)
            tip.SetDrawHeaderLine(True)
            tip.ApplyStyle("Office 2007 Blue")
            tip.SetDropShadow(True)
        return wrapped_call


"""Subclass of Preferences, which is generated by wxFormBuilder."""

import wx, wxx, app
import app.proCxxGUI as gui
from  ctx import theContext as context


# Implementing Preferences
class PreferencesDialog(app.Preferences):
    """
    This dialog is used for setup of common preferences.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """Initialization"""
        super(PreferencesDialog, self).__init__(parent)
        self._fontPane = gui.FontPreferences(self.m_auinotebook3)
        if wx.__version__ >= '3.0.0.0':
            self.m_auinotebook3.AddPage(self._fontPane, u"Fonts", False, wx.wx.NOT_FOUND)
        else:
            self.m_auinotebook3.AddPage(self._fontPane, u"Fonts", False, wx.NullBitmap)
        df = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        dfs = self.GetFontString(df)
        self._config = context.config
        dfs = self._config.Read("config/defaultFont", dfs)
        self._font = self.GetStringFont(dfs)

    def GetFontString(self, font):
        """Get info about current font"""
        return font.GetNativeFontInfo().ToString()

    def GetStringFont(self, string):
        """Get info current font"""
        f = wx.SystemSettings_GetFont(wx.SYS_DEFAULT_GUI_FONT)
        f.SetNativeFontInfoFromString(string)
        return f

    # Handlers for Preferences events.
    def OnInitDialog(self, event):
        """Dialog initialization"""
        self._fontPane.m_fontPicker.SetSelectedFont(self._font)
        # TODO: Implement OnInitDialog
        pass

    def OnOK(self, event):
        """OnOK handler"""
        dfs = self.GetFontString(self._fontPane.m_fontPicker.GetSelectedFont())
        self._config.Write("config/defaultFont", dfs)
        self._config.Flush()
        self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-

import model
import model.decorator as ctx
import app.resources as rc
import tran


class TaskFolder(model.TComponent):
    """Implements a Folder representation"""
    task_container = True

    # visual methods
    @tran.TransactionalMethod('move folder {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_task_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0
        new_cont = target.status_container
        if new_cont is not target.status_container:
            new_cont.SetStatus(self)
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    @property
    def status_container(self):
        """return the status container"""
        return self.parent.status_container

    def __init__(self, **kwargs):
        """Initialization"""
        super(TaskFolder, self).__init__(**kwargs)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folder")

    @property
    def bitmap_open_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folder_open")



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
import os
import wx
from  ctx import theContext as context
import model.file
import app.resources as rc


class check_updates(wx.PyEvtHandler):
    """Class for providing check_updates"""
    instance = None

    def __init__(self):
        """Initialize ast explorer"""
        super(check_updates, self).__init__()
        #create the menus
        self._menuid = context.frame.new_tool_id()
        self._imenu = wx.MenuItem(context.frame.menuTools, self._menuid, u"Check updates",
            u"check for existing updates", wx.ITEM_NORMAL)
        context.frame.AddToolsMenu('Check updates', self._imenu)
        #bind the menu handlers
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateCheckUpdates, id=self._menuid)
        self.Bind(wx.EVT_MENU, self.OnCheckUpdates, id=self._menuid)

    @classmethod
    def load(cls):
        """Setup tool for the environment"""
        return check_updates()

    def OnUpdateCheckUpdates(self, event):
        """Handle the update"""
        event.Enable(True)

    def OnCheckUpdates(self, event):
        """Handle the command"""
        self.doUpdate()

    @staticmethod
    def parse_pkginfo(rawstring):
        """Parses pkginfo and returns a ddictionary"""
        lines = rawstring.splitlines(False)
        stripper = lambda x: (x[0].strip().upper(), x[1].strip())
        return dict([stripper(x.split(':', 1)) for x in lines if ':' in x])

    @staticmethod
    def pkginfo_version(pkg_dict):
        """Get comparable package info from dict"""
        return [int(ns) for ns in pkg_dict['VERSION'].split('.')]

    def doUpdate(self, automode=False):
        """Do the real update process"""
        # Check internet connection
        import socket
        try:
            socket.gethostbyname("pypi.python.org")
        except:
            if not automode:
                wx.MessageBox("Unable to access pypi repository.\nIs internect accesible?", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        try:
            import urllib2
            raw = urllib2.urlopen('https://pypi.python.org/pypi?name=beatle&:action=display_pkginfo').read()
        except:
            if not automode:
                wx.MessageBox("We apologyze, but we can't access to beatle repository.\nPlease check again later.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        online_info = self.parse_pkginfo(raw)
        if 'VERSION' not in online_info:
            if not automode:
                wx.MessageBox("Missing version in beatle package", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        try:
            import subprocess as sp
            raw, err = sp.Popen(['pip', 'show', 'beatle'], stdout=sp.PIPE, stderr=sp.PIPE).communicate()
        except:
            if not automode:
                wx.MessageBox("We apologyze, but we can't read beatle pip info.", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        local_info = self.parse_pkginfo(raw)
        if 'VERSION' not in local_info:
            if not automode:
                wx.MessageBox("Missing version in beatle package", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        online_version = self.pkginfo_version(online_info)
        local_version = self.pkginfo_version(local_info)
        if local_version >= online_version:
            if not automode:
                wx.MessageBox("Beatle is up to date!", "Info",
                    wx.OK | wx.CENTER | wx.ICON_INFORMATION, context.frame)
            return
        else:
            vo = local_info['VERSION']
            vn = online_info['VERSION']
            option = wx.MessageBox(
                "There are a new version of Beatle.\n"
                "your current version is {vo}\n"
                "the new version is {vn}\n"
                "Do you want to update?".format(vo=vo, vn=vn), "Info",
                    wx.YES_NO | wx.CENTER | wx.ICON_QUESTION, context.frame)
            if option != wx.YES:
                return
        # attempt to do without sudo
        success = True
        if os.system("pip install beatle --upgrade"):
            #Ok, now with sudo
            if os.system("gksudo 'pip install beatle --upgrade'"):
                success = False
        if success:
            wx.MessageBox(
                "Beatle is now up to date! Please restart the\n"
                "application for using the new version.", "Info",
                wx.OK | wx.CENTER | wx.ICON_INFORMATION, context.frame)
        else:
            wx.MessageBox("Beatle is not updated.", "Info",
                wx.OK | wx.CENTER | wx.ICON_INFORMATION, context.frame)




"""Subclass of NewPyArgument, which is generated by wxFormBuilder."""

import re
import wx
import wxx
from activity.models.ui import ui as ui
import app.resources as rc


# Implementing NewArgument
class PyArgumentDialog(ui.NewPyArgument):
    """
    This dialog allows to specify name, and initialization for a method argument.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, method):
        """Initialization. The parent argument represents the mainWindow """
        super(PyArgumentDialog, self).__init__(parent)
        self.m_textCtrl6.SetFocus()
        self._container = method
        self._origin = None
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("py_argument"))
        self.SetIcon(icon)

    def CopyAttributes(self, argument):
        """Get the atributes"""
        argument._name = self._name
        argument._default = self._default
        argument.note = self._note

    def SetAttributes(self, argument):
        """Set the attributes"""
        self._origin = argument._name
        self.m_textCtrl6.SetValue(argument._name)
        self.m_textCtrl8.SetValue(argument._default)
        self.m_richText1.SetValue(argument.note)
        self.SetTitle("Edit argument")

    def Validate(self):
        """Validation"""
        self._name = self.m_textCtrl6.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Argument name must not be empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if self._name in ['args', 'kwargs']:
            wx.MessageBox("args and kwargs are special argument names", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if re.match("^[A-Za-z_][0-9A-Za-z_]*$", self._name) is None:
            wx.MessageBox("Argument name contains invalid characters", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        if self._origin != self._name:
            if self._container.ExistArgumentNamed(self._name):
                wx.MessageBox("Argument name already exists", "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, self)
                return False
        self._default = self.m_textCtrl8.GetValue()
        self._note = self.m_richText1.GetValue()
        return True

    def get_kwargs(self):
        """returns arguments suitable for creating new object"""
        return {'parent': self._container, 'name': self._name,
            'default': self._default, 'note': self._note}

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """ok event handler"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-


class clsspptdscrptr(object):
    """descriptor"""
    def __init__(self, getter, setter=None):
        """init"""
        self._getter = classmethod(getter)
        self._setter = setter and classmethod(setter)
        super(clsspptdscrptr, self).__init__()

    def __get__(self, inst, instype=None):
        """"""
        return self._getter.__get__(inst, instype or type(inst))()

    def __set__(self, inst, value):
        """"""
        assert self._setter
        return self._setter.__get__(inst, type(inst))(value)

    def setter(self, setter):
        """"""
        self._setter = classmethod(setter)
        return self


def classproperty(getter, setter=None):
    """classproperty decorator"""
    return clsspptdscrptr(getter, setter)


